#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
UCI regression datasets
"""
import os
import sys
import inspect
from copy import deepcopy

import numpy as np
import pandas as pd
from urllib import request

from kxy_datasets.base import UCIBaseRegression
from kxy_datasets.utils import extract_from_url



class Abalone(UCIBaseRegression):
	"""
	Reference: https://archive.ics.uci.edu/ml/datasets/Abalone
	"""
	def __init__(self):
		url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/abalone/abalone.data'
		df = pd.read_csv(url, names=['Sex', 'Length', 'Diameter', 'Height', 'Whole weight', \
			'Shucked weight', 'Viscera weight', 'Shell weight', 'Age'])
		df['Age'] += 1.5
		self._df = df
		y_columns = ['Age']
		x_columns = [_ for _ in df.columns if _ not in y_columns]
		self.x = df[x_columns].values
		self.y = df[y_columns].values
		self.x_columns = x_columns
		self.y_column = y_columns[0]



class AirFoil(UCIBaseRegression):
	"""
	Reference: https://archive.ics.uci.edu/ml/datasets/Airfoil+Self-Noise
	"""
	def __init__(self):
		url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00291/airfoil_self_noise.dat'
		df = pd.read_csv(url, sep='\t', names=['Frequency', 'Angle of Attack', 'Chord Length', 'Velocity', 'Displacement Thickness', 'Sound Pressure'])
		self._df = df
		y_columns = ['Sound Pressure']
		x_columns = [_ for _ in df.columns if _ not in y_columns]
		self.x = df[x_columns].values
		self.y = df[y_columns].values
		self.x_columns = x_columns
		self.y_column = y_columns[0]



class AirQuality(UCIBaseRegression):
	"""
	Reference: https://archive.ics.uci.edu/ml/datasets/Air+Quality
	"""
	def __init__(self, root_dir='./'):
		dataset_path = os.path.join(root_dir, self.name)
		url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00360/AirQualityUCI.zip'
		extract_from_url(url, dataset_path)
		file_path = os.path.join(dataset_path, 'AirQualityUCI.csv')
		df = pd.read_csv(file_path, sep=';', parse_dates=[0, 1])
		df.dropna(axis=0, how='all', inplace=True)
		df.dropna(axis=1, how='all', inplace=True)

		df.Date = (df.Date - df.Date.min()).astype('timedelta64[D]')  # Days as int
		df.Time = df.Time.apply(lambda x: int(x.split('.')[0]))  # Hours as int
		df['C6H6(GT)'] = df['C6H6(GT)'].apply(lambda x: float(x.replace(',', '.')))  # Target as float

		# Some floats are given with ',' instead of '.'
		df = df.applymap(lambda x: float(x.replace(',', '.')) if type(x) is str else x)  # Target as float

		df = df[df['C6H6(GT)'] != -200]  # Drop all rows with missing target values
		df.loc[df['CO(GT)'] == -200, 'CO(GT)'] = np.nan  # -200 means missing value, change to np.nan

		y_columns = ['C6H6(GT)']
		x_columns = [_ for _ in df.columns if _ not in y_columns]

		self.x = df[x_columns].values
		self.y = df[y_columns].values
		self._df = df
		self.x_columns = x_columns
		self.y_column = y_columns[0]



class BlogFeedback(UCIBaseRegression):
	"""
	Reference: https://archive.ics.uci.edu/ml/datasets/BlogFeedback
	"""
	def __init__(self, root_dir='./'):
		file_name = 'blogData_train.csv'
		dataset_path = os.path.join(root_dir, self.name)
		url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00304/BlogFeedback.zip'
		extract_from_url(url, dataset_path)

		test_dfs = []
		for fn in os.listdir(dataset_path):
			if 'blogData_test' not in fn:
				continue
			file_path = os.path.join(dataset_path, fn)
			test_dfs.append(pd.read_csv(file_path, header=None))
		df_test = pd.concat(test_dfs)

		file_path = os.path.join(dataset_path, file_name)
		df_train = pd.read_csv(file_path, header=None)
		y_columns = [280]
		x_columns = [_ for _ in df_train.columns if _ not in y_columns]
		df_train[y_columns[0]] = np.log(df_train[y_columns[0]] + 0.01)
		df_test[y_columns[0]]  = np.log(df_test[y_columns[0]] + 0.01)
		df = pd.concat([df_train, df_test])

		self.x_train = df_train[x_columns].values
		self.x_test  = df_test[x_columns].values
		self.x = df[x_columns].values

		self.y_train = df_train[y_columns].values
		self.y_test  = df_test[y_columns].values 
		self.y = df[y_columns].values

		self.x_columns = x_columns
		self.y_column = y_columns[0]



class CTSlices(UCIBaseRegression):
	"""
	Reference: https://archive.ics.uci.edu/ml/datasets/Relative+location+of+CT+slices+on+axial+axis
	"""
	def __init__(self, root_dir='./'):
		dataset_path = os.path.join(root_dir, self.name)
		url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00206/slice_localization_data.zip'
		extract_from_url(url, dataset_path)
		file_name = 'slice_localization_data.csv'
		file_path = os.path.join(dataset_path, file_name)
		df = pd.read_csv(file_path)
		self._df = df
		y_columns = ['reference']
		x_columns = [_ for _ in df.columns if _ not in y_columns]
		self.x = df[x_columns].values
		self.y = df[y_columns].values
		self.x_columns = x_columns
		self.y_column = y_columns[0]



class FacebookComments(UCIBaseRegression):
	"""
	Reference: https://archive.ics.uci.edu/ml/datasets/Facebook+Comment+Volume+Dataset
	"""
	def __init__(self, root_dir='./'):
		dataset_path = os.path.join(root_dir, self.name)
		url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00363/Dataset.zip'
		extract_from_url(url, dataset_path)
		dataset_path = os.path.join(dataset_path, 'Dataset')
		train_path = os.path.join(dataset_path, 'Training', 'Features_Variant_5.csv')
		test_path  = os.path.join(dataset_path, 'Testing', 'Features_TestSet.csv')
		df_train   = pd.read_csv(train_path, header=None)
		df_test    = pd.read_csv(test_path, header=None)
		df = pd.concat([df_train, df_test])

		y_columns = df_train.columns[-1:]
		x_columns = [_ for _ in df.columns if _ not in y_columns]
		self.x_train = df_train[x_columns].values
		self.x_test  = df_test[x_columns].values
		self.x = df[x_columns].values

		self.y_train = df_train[y_columns].values
		self.y_test  = df_test[y_columns].values 
		self.y = df[y_columns].values

		self.x_columns = x_columns
		self.y_column = y_columns[0]



class OnlineNews(UCIBaseRegression):
	"""
	Reference: https://archive.ics.uci.edu/ml/datasets/Online+News+Popularity
	"""
	def __init__(self, root_dir='./'):
		dataset_path = os.path.join(root_dir, self.name)
		url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00332/OnlineNewsPopularity.zip'
		extract_from_url(url, dataset_path)
		file_path = os.path.join(dataset_path, 'OnlineNewsPopularity', 'OnlineNewsPopularity.csv')
		df = pd.read_csv(file_path)
		df.drop(columns=['url', ' timedelta'], inplace=True)
		self._df = df
		y_columns = [' shares']
		x_columns = [_ for _ in df.columns if _ not in y_columns]
		self.x = df[x_columns].values
		self.y = df[y_columns].values
		self.x_columns = x_columns
		self.y_column = y_columns[0]



class Parkinson(UCIBaseRegression):
	"""
	Reference: http://archive.ics.uci.edu/ml/datasets/parkinsons
	"""
	def __init__(self):
		url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/' \
			  'parkinsons/telemonitoring/parkinsons_updrs.data'
		df = pd.read_csv(url)
		self._df = df
		y_columns = ['motor_UPDRS', 'total_UPDRS']
		x_columns = [_ for _ in df.columns if _ not in y_columns]
		self.x = df[x_columns].values
		self.y = df[y_columns].values
		self.x_columns = x_columns
		self.y_columns = y_columns
		self.y_column = y_columns[0]



class PowerPlant(UCIBaseRegression):
	"""
	Reference: https://archive.ics.uci.edu/ml/datasets/combined+cycle+power+plant
	"""
	def __init__(self, root_dir='./'):
		dataset_path = os.path.join(root_dir, self.name)
		url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00294/CCPP.zip'
		extract_from_url(url, dataset_path)
		file_path = os.path.join(dataset_path, 'CCPP', 'Folds5x2_pp.xlsx')
		df = pd.read_excel(file_path)
		self._df = df
		y_columns = ['PE']
		x_columns = [_ for _ in df.columns if _ not in y_columns]
		self.x = df[x_columns].values
		self.y = df[y_columns].values
		self.x_columns = x_columns
		self.y_column = y_columns[0]



class RealEstate(UCIBaseRegression):
	"""
	Reference: http://archive.ics.uci.edu/ml/datasets/Real+estate+valuation+data+set
	"""
	def __init__(self):
		url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00477/Real%20estate%20valuation%20data%20set.xlsx'
		df = pd.read_excel(url, index_col='No')
		self._df = df
		y_columns = ['Y house price of unit area']
		x_columns = [_ for _ in df.columns if _ not in y_columns]
		self.x = df[x_columns].values
		self.y = df[y_columns].values
		self.x_columns = x_columns
		self.y_column = y_columns[0]



class Superconductivity(UCIBaseRegression):
	"""
	Reference: https://archive.ics.uci.edu/ml/datasets/superconductivty+data
	"""
	def __init__(self, root_dir='./'):
		dataset_path = os.path.join(root_dir, self.name)
		url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00464/superconduct.zip'
		extract_from_url(url, dataset_path)
		file_path = os.path.join(dataset_path, 'train.csv')
		df = pd.read_csv(file_path)
		self._df = df
		y_columns = ['critical_temp']
		x_columns = [_ for _ in df.columns if _ not in y_columns]
		self.x = df[x_columns].values
		self.y = df[y_columns].values
		self.x_columns = x_columns
		self.y_column = y_columns[0]



class YachtHydrodynamics(UCIBaseRegression):
	"""
	Reference: http://archive.ics.uci.edu/ml/datasets/yacht+hydrodynamics
	"""
	def __init__(self):
		url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00243/yacht_hydrodynamics.data'
		df = pd.read_csv(url, sep='[ ]{1,2}', engine='python', \
						 names=['Longitudinal Position', 'Prismatic Coeefficient',\
								'Length-Displacement', 'Beam-Draught Ratio',\
								'Length-Beam Ratio', 'Froude Number',\
								'Residuary Resistance'])
		df.rename(columns={col: col.title() for col in df.columns}, inplace=True)
		self._df = df
		y_columns = ['Residuary Resistance']
		x_columns = [_ for _ in df.columns if _ not in y_columns]
		self.x = df[x_columns].values
		self.y = df[y_columns].values
		self.x_columns = x_columns
		self.y_column = y_columns[0]



class WhiteWineQuality(UCIBaseRegression):
	"""
	Reference: http://archive.ics.uci.edu/ml/datasets/Wine+Quality
	"""
	def __init__(self):
		url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/wine-quality/winequality-white.csv'
		df = pd.read_csv(url, sep=';')
		self._df = df
		y_columns = ['quality']
		x_columns = [_ for _ in df.columns if _ not in y_columns]
		self.x = df[x_columns].values
		self.y = df[y_columns].values
		self.x_columns = x_columns
		self.y_column = y_columns[0]



all_uci_regression_datasets = [\
	Abalone, AirFoil, AirQuality, BlogFeedback, CTSlices, FacebookComments, OnlineNews, PowerPlant, \
	Parkinson, RealEstate, Superconductivity, YachtHydrodynamics, WhiteWineQuality
]








#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
UCI classification datasets
"""
import os
import sys
import inspect

import pandas as pd
from unlzw import unlzw
from urllib import request

from kxy_datasets.base import UCIBaseClassification
from kxy_datasets.utils import extract_from_url


class Adult(UCIBaseClassification):
    """
    Reference: https://archive.ics.uci.edu/ml/datasets/adult
    """
    def __init__(self):
        url_train = 'http://archive.ics.uci.edu/ml/machine-learning-databases/adult/adult.data'
        url_test = 'http://archive.ics.uci.edu/ml/machine-learning-databases/adult/adult.test'
        df_train = pd.read_csv(url_train, names=['Age', 'Workclass', 'Fnlwgt', 'Education',\
            'Education Num', 'Marital Status',\
            'Occupation', 'Relationship', 'Race', 'Sex', \
            'Capital Gain', 'Capital Loss', 'Hours Per Week', \
            'Native Country', 'Income'])
        df_test = pd.read_csv(url_test, names=['Age', 'Workclass', 'Fnlwgt', 'Education',\
            'Education Num', 'Marital Status',\
            'Occupation', 'Relationship', 'Race', 'Sex', \
            'Capital Gain', 'Capital Loss', 'Hours Per Week', \
            'Native Country', 'Income'])
        df_test['Income'] = df_test['Income'].str.rstrip('.')
        df_test.index += len(df_train)
        df = pd.concat([df_train, df_test])
        self._df = df

        y_columns = ['Income']
        x_columns = [_ for _ in df.columns if _ not in y_columns]

        self.x_train = df_train[x_columns].values
        self.x_test  = df_test[x_columns].values
        self.x = df[x_columns].values

        self.y_train = df_train[y_columns].values
        self.y_test  = df_test[y_columns].values 
        self.y = df[y_columns].values

        self.classes = list(set(list(self.y.flatten())))
        self.x_columns = x_columns
        self.y_column = y_columns[0]



class APSFailure(UCIBaseClassification):
    """
    Reference: https://archive.ics.uci.edu/ml/datasets/APS+Failure+at+Scania+Trucks
    """
    def __init__(self):
        url_train = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00421/aps_failure_training_set.csv'
        url_test = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00421/aps_failure_test_set.csv'
        df_train = pd.read_csv(url_train, skiprows=20, na_values='na')
        df_test = pd.read_csv(url_test, skiprows=20, na_values='na')
        df = pd.concat([df_train, df_test])
        self._df = df

        y_columns = ['class']
        x_columns = [_ for _ in df.columns if _ not in y_columns]

        self.x_train = df_train[x_columns].values
        self.x_test  = df_test[x_columns].values
        self.x = df[x_columns].values

        self.y_train = df_train[y_columns].values
        self.y_test  = df_test[y_columns].values 
        self.y = df[y_columns].values

        self.classes = list(set(list(self.y.flatten())))
        self.x_columns = x_columns
        self.y_column = y_columns[0]



class Avila(UCIBaseClassification):
    """
    Reference: https://archive.ics.uci.edu/ml/datasets/Avila
    """
    def __init__(self, root_dir='./'):
        dataset_path = os.path.join(root_dir, self.name)
        url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00459/avila.zip'
        extract_from_url(url, dataset_path)
        file_path_train = os.path.join(dataset_path, 'avila', 'avila-tr.txt')
        file_path_test = os.path.join(dataset_path, 'avila', 'avila-ts.txt')

        columns = ['F%d' % i for i in range(1, 11)] + ['Class']
        df_train = pd.read_csv(file_path_train, names=columns)
        df_test = pd.read_csv(file_path_test, names=columns)
        df = pd.concat([df_train, df_test])
        self._df = df
        y_columns = ['Class']
        x_columns = [_ for _ in df.columns if _ not in y_columns]

        self.x_train = df_train[x_columns].values
        self.x_test  = df_test[x_columns].values
        self.x = df[x_columns].values

        self.y_train = df_train[y_columns].values
        self.y_test  = df_test[y_columns].values 
        self.y = df[y_columns].values

        self.classes = list(set(list(self.y.flatten())))
        self.x_columns = x_columns
        self.y_column = y_columns[0]


class BankMarketing(UCIBaseClassification):
    """
    Reference: https://archive.ics.uci.edu/ml/datasets/Bank+Marketing
    """
    def __init__(self, root_dir='./'):
        dataset_path = os.path.join(root_dir, self.name)
        url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00222/bank-additional.zip'
        extract_from_url(url, dataset_path)
        file_path = os.path.join(dataset_path, 'bank-additional', 'bank-additional-full.csv')
        df = pd.read_csv(file_path, sep=';')
        self._df = df
        y_columns = ['y']
        x_columns = [_ for _ in df.columns if _ not in y_columns]

        self.x = df[x_columns].values
        self.y = df[y_columns].values
        self.classes = list(set(list(self.y.flatten())))
        self.x_columns = x_columns
        self.y_column = y_columns[0]



class BankNote(UCIBaseClassification):
    """
    Reference: https://archive.ics.uci.edu/ml/datasets/banknote+authentication
    """
    def __init__(self):
        url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00267/' \
            'data_banknote_authentication.txt'
        df = pd.read_csv(url, names=['Variance', 'Skewness', 'Kurtosis', 'Entropy', 'Is Fake'])
        self._df = df
        y_columns = ['Is Fake']
        x_columns = [_ for _ in df.columns if _ not in y_columns]

        self.x = df[x_columns].values
        self.y = df[y_columns].values
        self.classes = list(set(list(self.y.flatten())))
        self.x_columns = x_columns
        self.y_column = y_columns[0]



class CardDefault(UCIBaseClassification):
    """
    Reference: https://archive.ics.uci.edu/ml/datasets/default+of+credit+card+clients
    """
    def __init__(self, root_dir='./'):
        url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00350/' \
              'default%20of%20credit%20card%20clients.xls'
        df = pd.read_excel(url, skiprows=1, index_col='ID')
        self._df = df
        y_columns = ['default payment next month']
        x_columns = [_ for _ in df.columns if _ not in y_columns]

        self.x = df[x_columns].values
        self.y = df[y_columns].values
        self.classes = list(set(list(self.y.flatten())))
        self.x_columns = x_columns
        self.y_column = y_columns[0]



class Landsat(UCIBaseClassification):
    """
    Reference: https://archive.ics.uci.edu/ml/datasets/Statlog+(Landsat+Satellite)
    """
    def __init__(self, root_dir='./'):
        url_train = 'http://archive.ics.uci.edu/ml/machine-learning-databases/statlog/satimage/sat.trn'
        url_test = 'http://archive.ics.uci.edu/ml/machine-learning-databases/statlog/satimage/sat.tst'
        df_train = pd.read_csv(url_train, sep=' ', header=None)
        df_test = pd.read_csv(url_test, sep=' ', header=None)
        df_test.index += len(df_train)
        df = pd.concat([df_train, df_test])
        y_columns = [36]
        x_columns = [_ for _ in df.columns if _ not in y_columns]

        self.x_train = df_train[x_columns].values
        self.x_test  = df_test[x_columns].values
        self.x = df[x_columns].values

        self.y_train = df_train[y_columns].values
        self.y_test  = df_test[y_columns].values 
        self.y = df[y_columns].values
        self.classes = list(set(list(self.y.flatten())))



class LetterRecognition(UCIBaseClassification):
    """
    Reference: https://archive.ics.uci.edu/ml/datasets/letter+recognition
    """
    def __init__(self):
        url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/letter-recognition/letter-recognition.data'
        df = pd.read_csv(url, header=None)
        y_columns = [0]
        x_columns = [_ for _ in df.columns if _ not in y_columns]

        self.x = df[x_columns].values
        self.y = df[y_columns].values
        self.classes = list(set(list(self.y.flatten())))



class MagicGamma(UCIBaseClassification):
    """
    Reference: https://archive.ics.uci.edu/ml/datasets/magic+gamma+telescope
    """
    def __init__(self):
        url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/magic/magic04.data'
        df = pd.read_csv(url, header=None)
        y_columns = [10]
        x_columns = [_ for _ in df.columns if _ not in y_columns]

        self.x = df[x_columns].values
        self.y = df[y_columns].values
        self.classes = list(set(list(self.y.flatten())))



class SensorLessDrive(UCIBaseClassification):
    """
    Reference: https://archive.ics.uci.edu/ml/datasets/dataset+for+sensorless+drive+diagnosis
    """
    def __init__(self):
        url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/00325/Sensorless_drive_diagnosis.txt'
        df = pd.read_csv(url, header=None, sep=' ')
        y_columns = [48]
        x_columns = [_ for _ in df.columns if _ not in y_columns]

        self.x = df[x_columns].values
        self.y = df[y_columns].values
        self.classes = list(set(list(self.y.flatten())))



class Shuttle(UCIBaseClassification):
    """
    Reference: https://archive.ics.uci.edu/ml/datasets/Statlog+(Shuttle)
    """
    def __init__(self):
        url_train = 'http://archive.ics.uci.edu/ml/machine-learning-databases/statlog/shuttle/shuttle.trn.Z'
        url_test = 'http://archive.ics.uci.edu/ml/machine-learning-databases/statlog/shuttle/shuttle.tst'
        os.makedirs(os.path.join('./', self.name), exist_ok=True)
        file_path_z = os.path.join(os.path.join('./', self.name), 'train.z')
        request.urlretrieve(url_train, file_path_z)
        file_path_csv = os.path.join(os.path.join('./', self.name), 'train.csv')
        with open(file_path_z, 'rb') as f_in:
            with open(file_path_csv, 'wb') as f_out:
                f_out.write(unlzw(f_in.read()))
        df_train = pd.read_csv(file_path_csv, header=None, sep=' ')
        df_test  = pd.read_csv(url_test, header=None, sep=' ')
        df = pd.concat([df_train, df_test])

        y_columns = [9]
        x_columns = [_ for _ in df.columns if _ not in y_columns]

        self.x_train = df_train[x_columns].values
        self.x_test  = df_test[x_columns].values
        self.x = df[x_columns].values

        self.y_train = df_train[y_columns].values
        self.y_test  = df_test[y_columns].values 
        self.y = df[y_columns].values
        self.classes = list(set(list(self.y.flatten())))



all_uci_classification_datasets = [\
    Adult, APSFailure, Avila, BankMarketing, BankNote, CardDefault, Landsat, LetterRecognition, \
    MagicGamma, SensorLessDrive, Shuttle
]






#!/usr/bin/env python
# -*- coding: utf-8 -*-
import abc

import numpy as np
import pandas as pd


class BaseDataset(abc.ABC):
    def __len__(self):
        return self.x.shape[0]

    def __getitem__(self, i):
        return self.x[i], self.y[i]

    @property
    def name(self):
        return getattr(self, '__name__', self.__class__.__name__)

    @property
    def num_features(self):
        return self.x.shape[1]

    @property
    def num_outputs(self):
        return self.y.shape[1] if len(self.y.shape) > 1 else 1

    @property
    def df(self):
        if getattr(self, '_df', None) is None:
            self.y_columns = ['y'] if self.num_outputs == 1 else ['y_%d' % i for i in range(self.num_outputs)]
            self.y_column = self.y_columns[0]
            self.x_columns = ['x_%d' % i for i in range(self.num_features)]
            columns = self.y_columns + self.x_columns
            return pd.DataFrame(np.concatenate([self.y, self.x], axis=1), columns=columns)

        else:
            return self._df



class BaseSyntheticClassification(BaseDataset):
    '''
    A classification dataset where the theoretical best performance achievable is known.
    '''
    @property
    def problem_type(self):
        return 'classification'

    @property
    def achievable_performance(self):
        return {'accuracy': self._achievable_accuracy}



class BaseRealClassification(BaseDataset):
    '''
    A classification dataset where the theoretical best performance achievable is not known.
    '''
    @property
    def problem_type(self):
        return 'classification'

    @property
    def num_classes(self):
        return len(self.classes)



class BaseSyntheticRegression(BaseDataset):
    '''
    A regression dataset where the theoretical best performance achievable is known.
    '''
    @property
    def problem_type(self):
        return 'regression'

    @property
    def achievable_performance(self):
        return {'rmse': self._achievable_rmse, 'r-squared': self._achievable_r_squared}



class BaseRealRegression(BaseDataset):
    '''
    A regression dataset where the theoretical best performance achievable is not known.
    '''
    @property
    def problem_type(self):
        return 'regression'


class KaggleBaseClassification(BaseRealClassification):
    @property
    def name(self):
        return getattr(self, '__name__', 'Kaggle' + self.__class__.__name__)


class KaggleBaseRegression(BaseRealRegression):
    @property
    def name(self):
        return getattr(self, '__name__', 'Kaggle' + self.__class__.__name__)


class UCIBaseClassification(BaseRealClassification):
    @property
    def name(self):
        return getattr(self, '__name__', 'UCI' + self.__class__.__name__)


class UCIBaseRegression(BaseRealRegression):
    @property
    def name(self):
        return getattr(self, '__name__', 'UCI' + self.__class__.__name__)







#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Kaggle regression datasets
"""
import os
import sys
import inspect

import pandas as pd
from unlzw import unlzw
from urllib import request

from kxy_datasets.base import KaggleBaseRegression
from kxy_datasets.utils import extract_from_url


class HousePricesAdvanced(KaggleBaseRegression):
	"""
	Reference: https://www.kaggle.com/c/house-prices-advanced-regression-techniques/
	"""
	def __init__(self):
		url = 's3://datasets.kxy.ai/regression/kaggle_house_prices_advanced_regression.csv'
		df = pd.read_csv(url)
		df.set_index('Id', inplace=True)
		self._df = df
		y_columns = ['SalePrice']
		x_columns = [_ for _ in df.columns if _ not in y_columns]
		self.x = df[x_columns].values
		self.y = df[y_columns].values
		self.x_columns = x_columns
		self.y_column = y_columns[0]




all_kaggle_regression_datasets = [\
	HousePricesAdvanced
]



# from .synthetic_classifications import *
# from .synthetic_regressions import *
# from .uci_classifications import *
# from .uci_regressions import *
# from .kaggle_classifications import *
# from .kaggle_regressions import *